dap = require('dap')

local opts = { noremap = true, silent = false }
-- vim.keymap.set('n', '<F5>', dap.continue(), opts)
vim.keymap.set('n', '<F5>', ":lua require('dap').continue()<CR>", opts)
vim.keymap.set('n', '<F9>', ":lua require('dap').toggle_breakpoint()<CR>", opts)
vim.keymap.set('n', '<F10>', ":lua require('dap').step_over()<CR>", opts)
vim.keymap.set('n', '<F11>', ":lua require('dap').step_into()<CR>", opts)
vim.keymap.set('n', '<S-F11>', ":lua require('dap').step_out()<CR>", opts)

local initDir = vim.api.nvim_list_runtime_paths()[1]
local masonpath = vim.fn.stdpath('data') .. '/mason'

-- PYTHON
local function getVenvSuffix()
  if vim.loop.os_uname().sysname == 'Linux' then
    return 'bin/python'
  elseif vim.loop.os_uname().sysname == 'Windows_NT' then
    return 'Scripts/python.exe'
  end
end

dap.adapters.python = {
  type = 'executable';
  command = masonpath .. '/packages/debugpy/venv/' .. getVenvSuffix();
  args = { '-m', 'debugpy.adapter' };
  options = {
    detached = true;
  };
}

local dap= require("dap")
local dapui = require("dapui")
dapui.setup()
dap.listeners.after.event_initialized["dapui_config"] = function()
  dapui.open()
end
dap.listeners.before.event_terminated["dapui_config"] = function()
  dapui.close()
end
dap.listeners.before.event_exited["dapui_config"] = function()
  dapui.close()
end

local function getPythonEnv()
  local venv = os.getenv("VIRTUAL_ENV")
  if venv ~= nil then
    return string.format("%s\\bin\\python.exe", venv)
  end

  local cwd = vim.fn.getcwd()
  if vim.fn.executable(cwd .. '/venv/' .. getVenvSuffix()) == 1 then
    return cwd .. '/venv/' .. getVenvSuffix()
  elseif vim.fn.executable(cwd .. '/.venv/' .. getVenvSuffix()) == 1 then
    return cwd .. '/.venv/' .. getVenvSuffix()
  else
    if vim.loop.os_uname().sysname == 'Linux' then
      return '/usr/bin/python'
    elseif vim.loop.os_uname().sysname == 'Windows_NT' then
      return os.getenv('SCOOP') .. '/apps/python/current/python.exe'
    end
  end
end

dap.configurations.python = {
  {
    type = 'python';
    request = 'launch';
    name = "Launch file with venv";
    justMyCode = false;
    program = "${file}";
    pythonPath = getPythonEnv
  },
}

-- require('dap-python').setup('C:\\Users\\oli\\AppData\\Local\\nvim\\venv_debugpy\\Scripts\\python')
-- table.insert(require('dap').configurations.python, {
--   -- type = 'python',
--   -- request = 'launch',
--   -- name = 'My custom launch configuration',
--   -- program = '${file}',
--   justMyCode = false
--   -- ... more options, see https://github.com/microsoft/debugpy/wiki/Debug-configuration-settings
-- })
-- require('dap-python').setup('C:/Users/oli/AppData/Local/nvim-data/mason/packages/debugpy/venv/Scripts/python.exe')

-- CPP
dap.adapters.codelldb = {
  type = 'server',
  port = "${port}",
  executable = {
    -- CHANGE THIS to your path!
    command = masonpath .. '/bin/codelldb.cmd',
    args = { '--port', '${port}' },

    -- On windows you may have to uncomment this:
    detached = false,
  }
}

dap.adapters.lldb = {
  type = 'executable',
  command = function()
    if vim.loop.os_uname().sysname == 'Linux' then
      return 'lldb-vscode'
    else
      return os.getenv('SCOOP') .. '/apps/llvm/current/bin/lldb-vscode.exe'
    end
  end,
  name = 'lldb'
}

dap.configurations.cpp = {
  {
    name = "Launch codelldb",
    type = "codelldb",
    request = "launch",
    program = function()
      return vim.fn.input('Path to executable: ', vim.fn.getcwd() .. '/build_nvim/', 'file')
    end,
    cwd = '${workspaceFolder}',
    stopOnEntry = true,
  },
}

dap.configurations.cpp = {
  {
    name = 'Launch lldb',
    type = 'lldb',
    request = 'launch',
    program = function()
      return vim.fn.input('Path to executable: ', vim.fn.getcwd() .. '/build_nvim', 'file')
    end,
    cwd = '${workspaceFolder}',
    stopOnEntry = false,
    args = {},

    -- 💀
    -- if you change `runInTerminal` to true, you might need to change the yama/ptrace_scope setting:
    --
    --    echo 0 | sudo tee /proc/sys/kernel/yama/ptrace_scope
    --
    -- Otherwise you might get the following error:
    --
    --    Error on launch: Failed to attach to the target process
    --
    -- But you should be aware of the implications:
    -- https://www.kernel.org/doc/html/latest/admin-guide/LSM/Yama.html
    -- runInTerminal = false,
  },
}

-- EXTENSIONS

require('nvim-dap-virtual-text').setup()
require('telescope').load_extension('dap')
require('cmp').setup.filetype({ 'dap-repl', 'dapui_watches', 'dapui_hover' }, {
  sources = {
    { name = 'dap' },
  },
})

